from openmdao.api import Problem, view_model, Component, view_connections, IndepVarComp, ScipyOptimizer, SqliteRecorder
from openmdao.core.driver import Driver
import numpy as np
import pandas as pd

import winsound
winsound.Beep(300,200)

from yamlparser import parsefile
from file_writer import write_files
from Aml import FmmWriter
from fedem import Fedem
from strain_rose import find_straine_rose_nodes, prep_fmm_for_gauge, create_strain_rosette
from desvars import create_desvars, Vectorizer
import parser_classes

import time


class Analyser(Component):

    def __init__(self):
        super().__init__()

        self.aml = FmmWriter()

        self.add_param('dict',val='', pass_by_obj=True)
        self.add_output('res', shape=1)

    def solve_nonlinear(self, params, unknowns, resids):
        print('DOING THE DEFEM')
        path = 'C:\\Users\\arnt\\Documents\\GitHub\\KBE\\library\\mechtimum-files\\out\\'
        bdf_filename = 'C:\\Temp\\nastran-data\\mechtimum-files - out\\link-0001.bdf'
        fmm_file = 'model.fmm'

        write_files(params['dict'], path=path)
        self.aml.write_fmm()
        time.sleep(1.0)
        # prep_fmm_for_gauge(path+fmm_file)
        fedem = Fedem(path, fmm_file)

        #create_strain_rosette(fedem, bdf_filename)

        #fedem.run(onlydyn=False, result_check_args=('rose', 'Stress tensor:TENSOR2:Von Mises', 28))
        fedem.run(onlydyn=False)

        y_result = fedem.get_res_on_bid(9, 'Position matrix:TMAT34:Position Y', 'y')
        # y_result.plot()
        x_result = fedem.get_res_on_bid(9, 'Position matrix:TMAT34:Position X', 'x')
        # x_result.plot()
        result = pd.concat([x_result, y_result], axis=1)
        result.columns = ['x', 'y']
        result = result.loc[:5.0]

        start = np.array([0.3, 0.3])
        end = np.array([0.0, 0.3])

        time_at_point = lambda path, point: path.apply(lambda df: np.linalg.norm(point - df), axis=1).idxmin()

        t_start = time_at_point(result, start)
        t_end = time_at_point(result, end)

        print('time: ', t_start, ' to ', t_end)

        path_of_interest = result.loc[t_start: t_end]

        # reparametrize the path in tau=0...1
        step_ends = path_of_interest.iloc[1:]
        step_starts = path_of_interest.iloc[:-1]
        step_starts.index = step_ends.index
        # step_ends.index = step_starts.index
        step_lengths_comps = step_ends - step_starts

        step_lengths = step_lengths_comps.apply(lambda df: np.linalg.norm(df), axis=1)
        path_length = step_lengths.sum()
        relative_length = (step_lengths.cumsum() / path_length).to_frame()

        relative_length = pd.DataFrame([0.0], index=[0.0]).append(relative_length)
        relative_length.columns = ['tau']
        relative_length.reset_index(inplace=True)
        relative_length.set_index('tau', inplace=True)
        relative_length.columns = ['time']

        psudo_time_step_sizes = relative_length

        reference_path = lambda tau: start + (end - start) * tau

        deviation = pd.merge(psudo_time_step_sizes.reset_index(), result.reset_index(), on='time')
        deviation['ref_path_x'] = deviation.apply(lambda df: reference_path(df['tau'])[0], axis=1)
        deviation['ref_path_y'] = deviation.apply(lambda df: reference_path(df['tau'])[1], axis=1)
        deviation['distance'] = deviation.apply(lambda df: np.linalg.norm(df[['x', 'y']] - reference_path(df['tau'])),
                                                axis=1)
        # deviation.set_index('tau').loc[:,'ref_path_x'].plot()
        deviation.set_index('time', inplace=True)
        deviation['distance'].plot()
        deviation

        obj = np.trapz(deviation['distance'], x=deviation['tau'])
        print('OBJ:', "{0:.6f}".format(obj))
        unknowns['res'] = obj
        winsound.Beep(300, 200)
        exit()
        fedem.end()

        # result = None
        # while True:
        #     result = fedem.get_res_on_bid(28, 'Stress tensor:TENSOR2:Von Mises', 'theRose')
        #     print(result.tail(3))
        #     if result.iloc[-1].loc['theRose:Stress tensor:TENSOR2:Von Mises'] != 0.0:
        #         print(result.tail())
        #         break
        #     time.sleep(1.0)
        #
        # print(result)
        # #exit()
        # stress_result = result
        #
        # #v = fedem.get_res_on_bid(9, 'Velocity:VEC3:Y', 'vel')
        #
        # result = fedem.get_res_on_bid(9, 'Position matrix:TMAT34:Position X', 'x')
        # result.columns = ['x']
        # print(result.head())
        # not_of_interest = result.loc[lambda df: df.x < 0, :]
        # settle_time = not_of_interest.index[0]
        # obj = abs(settle_time-2.0)
        # print('OBJ:', "{0:.6f}".format(settle_time))
        # print('ABS:', "{0:.6f}".format(obj))
        # #fedem.end()
        # unknowns['res'] = obj
        # print('the stress of pressure')
        # print(stress_result.max().loc['theRose:Stress tensor:TENSOR2:Von Mises'])
        # print('DONE THE DEFEM')



g, d = parsefile('cases/case1/case1.yaml')
analyser = Analyser()
g.add('analyser', analyser, promotes=['dict', 'res'])
prob = Problem(root=g)

prob.driver = ScipyOptimizer()
desvar_names = list(create_desvars(prob, add_to_driver=True))
cross_section_param = desvar_names[1]
g.connect(cross_section_param,
          [g.get_name_of(c)+'.dimensions' for
           c in g.subsystems(typ=parser_classes.Member,
                             recurse=True)])

#prob.root.add('design_param', IndepVarComp('c', 2005.0))
#prob.root.connect('design_param.c', 'couplings.NodeCoupling0.constant')


prob.root.deriv_options['type'] = 'fd'
#prob.root.deriv_options['form'] = 'backward'
prob.root.deriv_options['step_size'] = 0.01

#prob.driver = ScipyOptimizer()
prob.driver.options['optimizer'] = 'SLSQP'
prob.driver.options['tol'] = 1e-10
#prob.driver.add_desvar('design_param.c', lower=400.0, upper=4000.0)
prob.driver.add_objective('res', scaler=1.0)

recorder = SqliteRecorder('case1_opt.log')
recorder.options['record_params'] = True
recorder.options['record_metadata'] = True
recorder.options['record_derivs'] = True
prob.driver.add_recorder(recorder)

prob.setup()
view_model(prob, show_browser=False)
view_connections(g, show_browser=False)
#prob['x01'] = -0.05
#prob['x01'] = -0.05
print('running')
prob.run()
print('ran')
#print('the result is ', prob['res'], 'at ', prob['design_param.c'])

analyser.aml.aml.end()
prob.cleanup()
winsound.Beep(300,2000)

