from openmdao.api import CaseReader
import sys
import numpy as np

from openmdao.api import Group, Component, MetaModel, KrigingSurrogate, FloatKrigingSurrogate, Problem, ScipyOptimizer, IndepVarComp


class MechModel(Group):
    ''' FloatKriging gives responses as floats '''

    def __init__(self):
        super().__init__()

        # Create meta_model for f_x as the response
        inner_model = self.add("mech_model", MetaModel())
        inner_model.add_param('x', val=0.)
        inner_model.add_param('y', val=0.)
        inner_model.add_output('f', val=0., surrogate=FloatKrigingSurrogate())
        inner_model.add_output('c', val=0.0, surrogate=FloatKrigingSurrogate())

prob = Problem()
prob.root = MechModel()
root = prob.root

#prob.driver = ScipyOptimizer()
#prob.driver.options['optimizer'] = 'SLSQP'
#prob.driver.options['tol'] = 1e-10

root.add('p1', IndepVarComp('x', 0.1))
root.add('p2', IndepVarComp('y', 0.0))

root.connect('p1.x', 'mech_model.x')
root.connect('p2.y', 'mech_model.y')

prob.driver.add_desvar('p1.x', lower=0.05, upper=0.25)
prob.driver.add_desvar('p2.y', lower=-0.1, upper=0.1)
prob.driver.add_objective('mech_model.f')

prob.setup()

cr = CaseReader('cases/case1/case1_2_samples.log')
case_keys = list(cr.list_cases())
x = [cr.get_case(c)['x00'] for c in case_keys]
y = [cr.get_case(c)['x01'] for c in case_keys]
f = [cr.get_case(c)['res'] for c in case_keys]
c = [cr.get_case(c)['violation'] for c in case_keys]

prob['mech_model.train:x'] = x
prob['mech_model.train:y'] = y
prob['mech_model.train:f'] = f
prob['mech_model.train:c'] = c

#prob['mech_model.x'] = 0.151
#prob['mech_model.y'] = 0.01

# prob.run()
# print('Estimating ', prob['mech_model.f'], ' and ', prob['mech_model.c'])
# print('@:', prob['mech_model.x'], ',', prob['mech_model.y'])

deviations = []
cr = CaseReader('cases/case1/case1.log')
for case in cr.list_cases():
    prob['p1.x'] = cr.get_case(case)['x00']
    prob['p2.y'] = cr.get_case(case)['x01']
    prob.run()
    f_observed = cr.get_case(case)['res']

    deviations.append(prob['mech_model.f'] - f_observed)

print(deviations)
