from openmdao.api import Problem, view_model, Component, view_connections, IndepVarComp, ScipyOptimizer, SqliteRecorder
from openmdao.core.driver import Driver

from openmdao.drivers.latinhypercube_driver import OptimizedLatinHypercubeDriver

import sys
print(sys.path)

from yamlparser import parsefile

from desvars import create_desvars, Vectorizer
import parser_classes

import time

import rpyc
conn = rpyc.classic.connect("10.0.75.1")

class AnalysisWrapper(Component):

    def __init__(self):
        super().__init__()

        self.add_param('dict', val='', pass_by_obj=True)
        self.add_output('res', shape=1)
        self.add_output('violation', shape=1)

        #print('\n'.join(conn.modules.sys.path))
        #print(conn.modules.cases)

        self.remote_analyser = conn.modules['cases.case2.case2_1_win'].Analyser()

    def solve_nonlinear(self, params, unknowns, resids):
        print('calling the remote')
        response = self.remote_analyser.solve(params['dict'])
        print('remote returned w: ', response)
        for k, v in response.items():
            unknowns[k] = v


g, d = parsefile('cases/case1/case1.yaml')
analyser = AnalysisWrapper()
g.add('analyser', analyser, promotes=['dict', 'res', 'violation'])
prob = Problem(root=g)


prob.driver = OptimizedLatinHypercubeDriver(num_samples=128, seed=0, population=20, generations=4, norm_method=2)
desvar_names = list(create_desvars(prob, add_to_driver=True))
cross_section_param = desvar_names[1]
g.connect(cross_section_param,
          [g.get_name_of(c)+'.dimensions' for
           c in g.subsystems(typ=parser_classes.Member,
                             recurse=True)])

#prob.root.add('design_param', IndepVarComp('c', 2005.0))
#prob.root.connect('design_param.c', 'couplings.NodeCoupling0.constant')


prob.root.deriv_options['type'] = 'fd'
#prob.root.deriv_options['form'] = 'backward'
prob.root.deriv_options['step_size'] = 0.01


#prob.driver.add_desvar('design_param.c', lower=400.0, upper=4000.0)
prob.driver.add_objective('res', scaler=1.0)
prob.driver.add_constraint('violation', lower=0.0)

recorder = SqliteRecorder('/log/case2_1.log') #se relative path
recorder.options['record_params'] = True
recorder.options['record_metadata'] = True
recorder.options['record_derivs'] = True
prob.driver.add_recorder(recorder)

prob.setup()
view_model(prob, show_browser=False)
view_connections(g, show_browser=False)
#prob['x01'] = -0.05
#prob['x01'] = -0.05
try:
    print('running')
    prob.run()
    print('ran')

except Exception as e:
    print('run failed with ')
    print(e)


prob.cleanup()
analyser.remote_analyser.aml.aml.end()


