import winsound
winsound.Beep(300,200)

import numpy as np
import pandas as pd

from file_writer import write_files
from Aml import FmmWriter
from fedem import Fedem
from strain_rose import find_straine_rose_nodes, prep_fmm_for_gauge, create_strain_rosette

import sys

import time

class Analyser():

    def __init__(self):
        super().__init__()

        print('createing analyser')
        print('on path:')
        print('\n'.join(sys.path))

        self.aml = FmmWriter()


    def solve(self, params_dict):
        unknowns = {}
        print('DOING THE DEFEM')
        path = 'C:\\Users\\arnt\\Documents\\GitHub\\KBE\\library\\mechtimum-files\\out\\'
        bdf_filename = 'C:\\Temp\\nastran-data\\mechtimum-files - out\\link-0001.bdf'
        fmm_file = 'model.fmm'

        write_files(params_dict, path=path)
        self.aml.write_fmm()
        time.sleep(1.0)
        # prep_fmm_for_gauge(path+fmm_file)
        fedem = Fedem(path, fmm_file)

        #create_strain_rosette(fedem, bdf_filename)

        #fedem.run(onlydyn=False, result_check_args=('rose', 'Stress tensor:TENSOR2:Von Mises', 28))
        try:
            fedem.run(onlydyn=False)
        except RuntimeError as e:
            print('Fedem run w timeout on solve, lets hope the results ar OK')
            print(e)

        y_result = fedem.get_res_on_bid(9, 'Position matrix:TMAT34:Position Y', 'y')
        # y_result.plot()
        x_result = fedem.get_res_on_bid(9, 'Position matrix:TMAT34:Position X', 'x')
        # x_result.plot()
        result = pd.concat([x_result, y_result], axis=1)
        result.columns = ['x', 'y']
        result = result.loc[:5.0]

        start = np.array([0.3, 0.3])
        end = np.array([0.0, 0.3])

        time_at_point = lambda path, point: path.apply(lambda df: np.linalg.norm(point - df), axis=1).idxmin()

        t_start = time_at_point(result, start)
        t_end = time_at_point(result, end)

        print('time: ', t_start, ' to ', t_end)

        path_of_interest = result.loc[t_start: t_end]

        # reparametrize the path in tau=0...1
        step_ends = path_of_interest.iloc[1:]
        step_starts = path_of_interest.iloc[:-1]
        step_starts.index = step_ends.index
        # step_ends.index = step_starts.index
        step_lengths_comps = step_ends - step_starts

        step_lengths = step_lengths_comps.apply(lambda df: np.linalg.norm(df), axis=1)
        path_length = step_lengths.sum()
        relative_length = (step_lengths.cumsum() / path_length).to_frame()

        relative_length = pd.DataFrame([0.0], index=[0.0]).append(relative_length)
        relative_length.columns = ['tau']
        relative_length.reset_index(inplace=True)
        relative_length.set_index('tau', inplace=True)
        relative_length.columns = ['time']

        psudo_time_step_sizes = relative_length

        reference_path = lambda tau: start + (end - start) * tau

        deviation = pd.merge(psudo_time_step_sizes.reset_index(), result.reset_index(), on='time')
        deviation['ref_path_x'] = deviation.apply(lambda df: reference_path(df['tau'])[0], axis=1)
        deviation['ref_path_y'] = deviation.apply(lambda df: reference_path(df['tau'])[1], axis=1)
        deviation['distance'] = deviation.apply(lambda df: np.linalg.norm(df[['x', 'y']] - reference_path(df['tau'])),
                                                axis=1)
        # deviation.set_index('tau').loc[:,'ref_path_x'].plot()
        deviation.set_index('time', inplace=True)
        deviation['distance'].plot()
        deviation

        deviation['violation'] = deviation.apply(lambda df: min(0, df['y'] - reference_path(df['tau'])[1]), axis=1)
        deviation['violation'].plot()
        violation_constraint = np.trapz(deviation['violation'], x=deviation['tau'])

        obj = np.trapz(deviation['distance'], x=deviation['tau'])
        print('OBJ:', "{0:.7f}".format(obj), ' at constraint value {0:.8f}'.format(violation_constraint))
        unknowns['res'] = obj
        unknowns['violation'] = violation_constraint
        winsound.Beep(300, 200)

        print('closing FEDEM')
        fedem.end()
        return unknowns

