from openmdao.api import CaseReader
import sys
import numpy as np
import time

from openmdao.api import Group, Component, MetaModel, KrigingSurrogate, FloatKrigingSurrogate, Problem, ScipyOptimizer, IndepVarComp, SqliteRecorder
from openmdao.api import pyOptSparseDriver
s = 1000.0

class MechModel(Group):
    ''' FloatKriging gives responses as floats '''

    def __init__(self):
        super().__init__()

        # Create meta_model for f_x as the response
        inner_model = self.add("mech_model", MetaModel())
        inner_model.add_param('x', val=0.)
        inner_model.add_param('y', val=0.)
        inner_model.add_output('f', val=0., surrogate=FloatKrigingSurrogate())
        inner_model.add_output('c', val=0.0, surrogate=FloatKrigingSurrogate())

prob = Problem()
prob.root = MechModel()
root = prob.root

prob.driver = pyOptSparseDriver()
#prob.driver.options['optimizer'] = 'IPOPT'
#prob.driver.options['tol'] = 1e-10

root.add('p1', IndepVarComp('x', 0.1))
root.add('p2', IndepVarComp('y', 0.0))

root.connect('p1.x', 'mech_model.x')
root.connect('p2.y', 'mech_model.y')

prob.driver.add_desvar('p1.x', lower=s*0.05, upper=s*0.25)
prob.driver.add_desvar('p2.y', lower=s*-0.1, upper=s*0.1)
prob.driver.add_objective('mech_model.f')
prob.driver.add_constraint('mech_model.c', lower=0.0)




#print('Estimating ', prob['mech_model.f'], ' and ', prob['mech_model.c'])
#print('@:', prob['mech_model.x'], ',', prob['mech_model.y'])
#prob.setup()

cr = CaseReader('cases/case1/case1_2_samples2.log')
case_keys = list(cr.list_cases())
x = np.array([cr.get_case(c)['x00'] for c in case_keys])
y = np.array([cr.get_case(c)['x01'] for c in case_keys])
f = np.array([cr.get_case(c)['res'] for c in case_keys])
c = np.array([cr.get_case(c)['violation'] for c in case_keys])

recorder = SqliteRecorder('/log/case2_2.log')
recorder.options['record_params'] = True
recorder.options['record_metadata'] = True
recorder.options['record_derivs'] = True
prob.driver.add_recorder(recorder)


for alg in ['SNOPT', 'IPOPT', 'SLSQP']:
    print('Running with ', alg)
    prob.driver.options['optimizer'] = alg


    prob.setup()
    prob['p1.x'] = s * 0.20
    prob['p2.y'] = s * 0.05

    prob['mech_model.train:x'] = s * x
    prob['mech_model.train:y'] = s * y
    prob['mech_model.train:f'] = s * f
    prob['mech_model.train:c'] = s * c

    prob.run()
    time.sleep(1.0)

prob.cleanup()



