from openmdao.drivers.latinhypercube_driver import OptimizedLatinHypercubeDriver

from yamlparser import parsefile
from file_writer import write_files
from Aml import FmmWriter
from fedem import Fedem
from displacement_control import add_motion
from openmdao.api import Problem, view_model, Component, view_connections, IndepVarComp, ScipyOptimizer, SqliteRecorder
from openmdao.core.driver import Driver
import numpy as np
import pandas as pd
from pprint import pprint
from desvars import create_desvars
import time

from cases.case3.design_space_transform import DesignSpaceTransformer

start = time.time()

class Analyser(Component):

    def __init__(self):
        super().__init__()

        self.aml = FmmWriter()

        self.add_param('dict',val='', pass_by_obj=True)

        self.result_names = ['t_p', 'x_p', 'y_p','v_x_p', 'v_y_p',
                             't_d', 'x_d', 'y_d', 'v_x_d', 'v_y_d', 'a_x_d', 'a_y_d',
                             'phi_p', 'phi_d', 'phi_dot_d', 'phi_dot_p']

        for name in self.result_names:
            self.add_output(name, shape=1)

    def solve_nonlinear(self, params, unknowns, resids):
        print('DOING THE DEFEM')
        path = 'C:\\Users\\arnt\\Documents\\GitHub\\KBE\\library\\mechtimum-files\\out\\'
        bdf_filename = 'C:\\Temp\\nastran-data\\mechtimum-files - out\\link-0001.bdf'
        fmm_file = 'model.fmm'
        write_files(params['dict'], path=path)
        self.aml.write_fmm()
        add_motion(path+fmm_file)
        time.sleep(1.0)

        #write Nan to the res a sa default
        for k in self.result_names:
            unknowns[k] = np.nan

        try:
            fedem = Fedem(path, fmm_file, duration=6.0, step_size=0.001)
        except ValueError as e:
            print('fedem start error')
            print(e)
            return

        try:
            fedem.run(onlydyn=True)
        except RuntimeError as e:
            print(e)
            res = fedem.get_res_on_bid(9, 'Position matrix:TMAT34:Position X', 'coupler')
            print(res.head((2)))
            try:
                print('TIME', res.index[-1])
                print(res.iloc[-1])
            except Exception:
                print('this is probably really bad')

        #lets hope there is some interesting results available:
        finally:
            try:
                res = self.extract_results(fedem)

                print('-'*10, ' results', '-'*10)
                pprint(res)
                for k, v in res.items():
                    unknowns[k] = v
                print('-'*30)
            except Exception as e:
                #it did not work, lets see whats wrong
                print(e)

            finally:
                fedem.end()


    @staticmethod
    def extract_results(fedem):
        sim_offset, sim_length = 3.1, 1.2
        x_end, y_end = 0.7925, 0.274
        x_start, y_start = 0.457, -0.305
        phi_offset = np.pi/4

        coupler_path = pd.concat([fedem.get_res_on_bid(9, 'Position matrix:TMAT34:Position X', 'coupler'),
                                  fedem.get_res_on_bid(9, 'Position matrix:TMAT34:Position Y', 'coupler')],
                                 axis=1)
        coupler_path = coupler_path.loc[sim_offset:sim_offset + sim_length]
        coupler_path.columns = ['x', 'y']

        time_at_point = lambda path, point: path.apply(lambda df: np.linalg.norm(point - df), axis=1).idxmin()

        t_start = time_at_point(coupler_path[['x', 'y']], np.array([x_start, y_start]))
        t_end = time_at_point(coupler_path[['x', 'y']], np.array([x_end, y_end]))
        print('pick up at ', t_start)
        print('deposit at ', t_end)

        coupler_path_high_order = [
            fedem.get_res_on_bid(9, 'Velocity:VEC3:X', 'coupler'),
            fedem.get_res_on_bid(9, 'Velocity:VEC3:Y', 'coupler'),
            fedem.get_res_on_bid(9, 'Acceleration:VEC3:X', 'coupler'),
            fedem.get_res_on_bid(9, 'Acceleration:VEC3:Y', 'coupler'),
            fedem.get_res_on_bid(9, 'Position matrix:TMAT34:Euler Angle ZYX Z', 'coupler'),
            fedem.get_res_on_bid(18, 'Angular velocity:ROT3:Z', 'coupler')
        ]

        path_derivs = pd.concat(coupler_path_high_order, axis=1)
        path_derivs.columns = ['v_x', 'v_y', 'a_x', 'a_y', 'phi', 'phi_dot']
        #path_derivs = path_derivs.loc[[t_start, t_end]]
        print(coupler_path.head())
        return {'t_p': t_start,
               'x_p': coupler_path.loc[[t_start]]['x'],
               'y_p': coupler_path.loc[[t_start]]['y'],
               'v_x_p': path_derivs.loc[[t_start]]['v_x'],
               'v_y_p': path_derivs.loc[[t_start]]['v_y'],
               't_d': t_end,
               'x_d': coupler_path.loc[[t_end]]['x'],
               'y_d': coupler_path.loc[[t_end]]['y'],
               'v_x_d': path_derivs.loc[[t_end]]['v_x'],
               'v_y_d': path_derivs.loc[[t_end]]['v_y'],
               'a_x_d': path_derivs.loc[[t_end]]['a_x'],
               'a_y_d': path_derivs.loc[[t_end]]['a_y'],
               'phi_p': path_derivs.loc[[t_start]]['phi'],
               'phi_d': path_derivs.loc[[t_start]]['phi'],
               'phi_dot_p':  path_derivs.loc[[t_start]]['phi_dot'],
               'phi_dot_d':  path_derivs.loc[[t_end]]['phi_dot']
                }

class AngleCombiner(Component):

    def __init__(self):
        super().__init__()

        self.add_param('alpha_reduced', val=np.array([2.365, 1.2217, 2.356]))
        self.add_output('alpha', val=np.array([2.365, 1.2217, 2.356, 0.7941]))

    def solve_nonlinear(self, params, unknowns, resids):
        unknowns['alpha'][0] = params['alpha_reduced'][0]
        unknowns['alpha'][1] = params['alpha_reduced'][1]
        unknowns['alpha'][2] = params['alpha_reduced'][2]

        #"Locked":
        #unknowns['alpha'][3] = params['alpha_reduced'][3]

g, d = parsefile('cases/case3/case3.yaml')
analyser = Analyser()
g.add('analyser', analyser, promotes=['dict'])
g.add('var_transformator', DesignSpaceTransformer())
g.add('angle_combiner', AngleCombiner())
prob = Problem(root=g)

for i in [0, 3, 4, 5, 6]:
    g.connect('var_transformator.mapped_points.P{}'.format(i),
              'points.P{}.position'.format(i))

g.connect('angle_combiner.alpha', 'var_transformator.transformations.alpha')

g.add('descomp1', IndepVarComp('D', 0.1364), promotes=['D'])
g.connect('D', 'var_transformator.constraints.D')
g.add('descomp2', IndepVarComp('len0', .25), promotes=['len0'])
g.connect('len0', 'var_transformator.constraints.len0')

g.add('descomp3', IndepVarComp('len2', 0.1940), promotes=['len2'])
g.connect('len2', 'var_transformator.constraints.len2')

g.add('descomp4', IndepVarComp('alpha_reduced',
                               val=np.array([2.365, 1.2217, 2.356]))
      , promotes=['alpha_reduced'])
g.connect('alpha_reduced', 'angle_combiner.alpha_reduced')

prob.driver = OptimizedLatinHypercubeDriver(num_samples=256, seed=0, population=50, generations=8, norm_method=2)
prob.driver.add_desvar('D', lower=0.135, upper=0.7)
prob.driver.add_desvar('len0', lower=0.25, upper=0.75)
prob.driver.add_desvar('len2', lower=0.1, upper=0.4)
prob.driver.add_desvar('alpha_reduced',
                       lower=np.array([2.0, 1.0, 2.0]),
                       upper=np.array([3.0, 2.0, 3.0]))

recorder = SqliteRecorder('cases/case3/case3_doe9_6dof.log')
recorder.options['record_params'] = True
recorder.options['record_metadata'] = True
recorder.options['record_derivs'] = True
prob.driver.add_recorder(recorder)


prob.setup()
#prob['points.P3.position'] = np.array([0.05837822,  0.4737455])
#prob['points.P4.position'] = np.array([0.50589701,  0.1])
view_model(prob, show_browser=False)
view_connections(g, show_browser=False)
prob.run()
analyser.aml.aml.end()
prob.cleanup()
print( 'It took', time.time()-start, 'seconds.')