import seaborn as sns
from openmdao.api import CaseReader as CR
import numpy as np
import pandas as pd

from openmdao.api import Group, Problem, Component, MetaModel, KrigingSurrogate,\
    FloatKrigingSurrogate, NearestNeighbor, ScipyOptimizer, IndepVarComp, ResponseSurface,\
    view_model, view_connections

cr = CR('case3_doe4_2dof.log')
c = cr.get_case(0)
print(cr.list_cases())

samples = [c for c in cr.list_cases() if not np.isnan(cr.get_case(c)['analyser.x_p'])]
print(len(samples))

res_names = ['t_p', 'x_p', 'y_p','v_x_p', 'v_y_p',
                             't_d', 'x_d', 'y_d', 'v_x_d', 'v_y_d', 'a_x_d', 'a_y_d',
                             'phi_p', 'phi_d', 'phi_dot_d', 'phi_dot_p']

d = {k: [cr.get_case(s)['analyser.' + k] for s in samples] for k in res_names}
df = pd.DataFrame(d)

point_names = ['points.P{}.position'.format(i) for i in [3, 4]]
d = {k: [cr.get_case(s).parameters[k][:2] for s in samples] for k in point_names}
d2 = {k: [cr.get_case(s).parameters[k][:2] for s in cr.list_cases()] for k in point_names}

all_points_df = pd.DataFrame(d2)

point_df = pd.DataFrame(d)

param_names = [s.split('.')[1] for s in point_names]
print(param_names)

res_names = ['t_p', 'x_p', 'y_p', 'v_x_p', 'v_y_p',
             't_d', 'x_d', 'y_d', 'v_x_d', 'v_y_d', 'a_x_d', 'a_y_d',
             'phi_p', 'phi_d', 'phi_dot_d', 'phi_dot_p']

active_res_names = res_names
active_res_names.remove('t_d')

x_min, x_max = np.array([0.0, 0.4, 0.4, 0.1]), np.array([0.2, 0.6, 0.6, 0.2])
#x4_min, x4_max = np.array([0.4, 0.1]), np.array([0.6, 0.2])


class ResultAnalyser(Component):
    def __init__(self):
        super().__init__()

        self.deriv_options['type'] = 'fd'
        self.deriv_options['form'] = 'central'

        for param in active_res_names:
            self.add_param(param, val=0.0)

        self.add_output('res', 0.0)
        self.distances = None

    def solve_nonlinear(self, params, unknowns, resids):
        # print('solving', params['a_y_d'])
        # print({k: params[k] for k in active_res_names})
        self.distances = np.array([
            params['a_x_d'] - 2.7084, #2.423942,
            params['a_y_d'] - -2.7084,#-2.626733,
            #params['phi_d'] - 0.0,#-1.737534,
            #params['phi_dot_d'] - np.pi,#-3.173199,
            # params['phi_dot_p'] - 0.499949,
            # params['phi_p'] - -1.737534,
            # t_d - 4.020000,
            # params['t_p'] - 3.530000,
            params['v_x_d'] - -0.8621,#-0.873219,
            # params['v_x_p'] - -0.098792,
            params['v_y_d'] - -0.8621,#-0.859992,
            # params['v_y_p'] - 0.104660,
            # params['x_d'] - 0.794198,
            # params['x_p'] - 0.456028,
            # params['y_d'] - 0.276021,
            # params['y_p'] - -0.30442,
            0.0
        ])

        unknowns['res'] = np.linalg.norm(self.distances)


class dMetaModel(MetaModel):
    def solve_nonlinear(self, params, unknowns, resids):
        print('mm: ', params['P3'], params['P4'])
        super().solve_nonlinear(params, unknowns, resids)


class MechanismMM(Group):
    def __init__(self):
        super().__init__()

        steph_mm = self.add('steph_mm', MetaModel())

        # steph_mm.deriv_options['type'] = 'fd'
        # steph_mm.deriv_options['step_size'] = 10.0
        # steph_mm.deriv_options['form'] = 'central'
        # steph_mm.deriv_options['step_calc'] = 'absolute'


        steph_mm.add_param('x', val=np.array([0.0, 0.0, 0.0, 0.0]))


        for output_name in active_res_names:
            # steph_mm.add_output(output_name,
            #                    val=0.0,
            #                    surrogate=KrigingSurrogate(nugget= 1e-4))
            #steph_mm.add_output(output_name, val=0.0, surrogate=KrigingSurrogate())
            steph_mm.add_output(output_name, val=0.0, surrogate=NearestNeighbor('linear'))
            #steph_mm.add_output(output_name, val=0.0, surrogate=NearestNeighbor('rbf'))
            #steph_mm.add_output(output_name, val=0.0, surrogate=NearestNeighbor('weighted'))
            #steph_mm.add_output(output_name, val=0.0, surrogate=ResponseSurface())


prob = Problem()
prob.root = MechanismMM()
prob.root.add('res_analyser', ResultAnalyser())
for res in active_res_names:
    prob.root.connect('steph_mm.' + res,
                      'res_analyser.' + res)

# prob.root.add('P3', IndepVarComp('x', np.array([0.1085, 0.4935])))
prob.root.add('P', IndepVarComp('x', np.array([0.18, 0.45, 0.55, 0.16])))
prob.root.connect('P.x', 'steph_mm.x')

prob.driver = ScipyOptimizer()
prob.driver.options['optimizer'] = 'SLSQP'
prob.driver.options['tol'] = float(1e-10)
prob.driver.add_desvar('P.x',
                       lower=x_min,
                       upper=x_max,  # )#,
                       adder=-x_min,
                       scaler=1 / x_max)
prob.driver.add_objective('res_analyser.res', scaler=1.0)

#prob.root.deriv_options['type'] = 'fd'
#prob.root.deriv_options['step_size'] = 0.05
# prob.root.deriv_options['step_calc'] = 'relative'


prob.setup()
#view_model(prob)
#view_connections(prob)

# training samples
print('training', 'steph_mm.train:x')
desvar_samples = [np.concatenate((p3, p4)) for p3, p4 in
           zip(point_df['points.P3.position'].tolist(), point_df['points.P4.position'].tolist())]
#print('using ', len(desvar_samples), 'points')
#pprint(points)
prob['steph_mm.train:x'] = desvar_samples

# training results
for res in active_res_names:
    res_list = df[res].tolist()
    #print('using ', len(res_list), 'results')
    # pprint(res_list)
    prob['steph_mm.train:' + res] = df[res].tolist()

prob.run()

print('Last iter @ desvar_point:', prob['steph_mm.x'],
      '\nwith ', prob['res_analyser.res'])
print('PREDICTIONS:\n')
for res in active_res_names:
    print(res, ': ', prob['steph_mm.' + res])

print(prob.find_subsystem('res_analyser').distances)

print('-' * 60)
from pprint import pprint

J = prob.calc_gradient(['P.x'], ['steph_mm.' + res for res in active_res_names])
J2 = prob.calc_gradient(['P.x'], ['res_analyser.res'])
print(J)
print(J2)

