from openmdao.api import Component, IndepVarComp
from parser_classes import AggregatedComp
import numpy as np

class Vectorizer(Component):

    def __init__(self, group, name, comp, *, driver=None):
        super().__init__()

        comp_param, default, minimums, maximums = comp.desvar

        self.desvar_names = []
        self.default=np.array(default)
        self.name = name
        self.add_output('vec', val=self.default)

        self.min = [x for x in minimums if x != '_']
        self.max = [x for x in maximums if x != '_']
        self.idxes = [i for i, x in enumerate(minimums) if x != '_']
        print('indexes:', self.idxes)

        for i in self.idxes:
            desvar_name = name+str(i)
            self.desvar_names.append(desvar_name)
            print('THe var ', desvar_name, ' is ', default[1])
            indep = IndepVarComp(desvar_name, default[i])
            self.add_param(desvar_name, val=default[i])
            group.add(name+'_indep'+str(i),
                      indep,
                      promotes=[desvar_name])
            if driver:
                print('adding desvar')
                driver.add_desvar(desvar_name,
                                  lower=self.min[i],
                                  upper=self.max[i])
            else:
                print('nod desvar')

        group.add(name, self, promotes=self.desvar_names)
        group.connect(self.output_name(),
                      '{}.{}'.format(group.get_name_of(comp), comp_param))

    def solve_nonlinear(self, params, unknowns, resids):
        #print('vectoizing ', [(k, v['val']) for k, v in params.items()])
        out = self.default
        for i, name in zip(self.idxes, self.desvar_names):
            out[i] = params[name]
        print('vectorzed', out)
        unknowns['vec'] = out

    def output_name(self):
        return self.name+'.vec'


def add_desvar(comp, prob, name, add_to_driver):
    driver = prob.driver if add_to_driver else None

    print('adding ', comp.desvar)
    #default, min, max = comp.desvar
    if len(comp.desvar[1]):
        vec = Vectorizer(prob.root, name, comp,
                         driver=driver)
        return vec.output_name()
    else:
        raise ValueError('Desvar not a list')


def create_desvars(prob, add_to_driver=True):
    g = prob.root

    desvar_comps = []
    for c in g.components(recurse=True, ):
        if isinstance(c, AggregatedComp) and c.desvar:
            desvar_comps.append(c)

    for i, c in enumerate(desvar_comps):
        yield add_desvar(c, prob, 'x{}'.format(i),
                         add_to_driver=add_to_driver)
