import win32com.client
import pythoncom
from win32com.client import VARIANT
import time
import pandas as pd
import shutil
import os
import subprocess


class Fedem:

    def __init__(self, path, fmm, duration=10.0, step_size=0.01):
        self.timeout = 60

        #Clean up old stuff

        try:
            if os.path.isdir(path + 'model_RDB'):
                shutil.rmtree(path + 'model_RDB')
                print('deleted RDB')
            else:
                print('no rdb')
        except Exception as e:
            print('delition failed')
            print(e)

        self.fedApp = Fedem._fedem_at_file(path+fmm)
        self.solver = self.fedApp.Solvers

        print('at file:\n', self.fedApp.ModelFileName)

        self.solver.StopTime = duration
        self.solver.TimeIncrement = step_size


        #self.run()

        #print(self._get_results('top', 'Position matrix:TMAT34:Position X'))

    def run(self, onlydyn=True, result_check_args=('top', 'Position matrix:TMAT34:Position X', 20)):
        print('solving')
        self.solver.EraseSimulationResults()
        if onlydyn:
            self.solver.SolveDynamics()
        else:
            self.solver.SolveAll()

        prev_observed_time = -1.0

        for _ in range(self.timeout):
            data = self._get_results(*result_check_args[:2], base_id=result_check_args[2])
            if not data.empty:
                print(data.index[-1])
                last_step = data.index[-1]
                almost_end = self.solver.StopTime - 3 * self.solver.TimeIncrement

                if not last_step > prev_observed_time:
                    print('fedem time not progressing(@ ', last_step, 'prev: ', prev_observed_time)
                    raise RuntimeError('FEDEM max iter?' )
                prev_observed_time = last_step

                print('last Timestep: ', last_step, 'and end at ', almost_end)
                if last_step > almost_end:
                    print('solved')
                    return

            time.sleep(1.0)

        raise RuntimeError('FEDEM TIMEOUT?')

    def create_strain_rosette(self, nodes, link_id):
        assert len(nodes) == 3
        link = self.fedApp.FindByBaseID(link_id)
        self.fedApp.CreateStrainRosette(1, link, *nodes, -1)

    def get_bfd_path(self, bid):
        com_link = self.fedApp.FindByBaseId(bid)
        assert com_link
        link = self.fedApp.GetFmObject(com_link)
        path = link.GetValue('ORIGINAL_FE_FILE')
        print('thte path for link', link.BaseID, 'is \n', path)
        return path


    def end(self):
        self.fedApp.Exit()

    def get_res_on_bid(self, base_id, field, name):
        return self._get_results(name, field, base_id=base_id)


    @staticmethod
    def _double(d):
        return VARIANT(pythoncom.VT_R8 | pythoncom.VT_BYREF, d)

    def _get_results(self, location, field, *, base_id=None):
        try:
            start_time = self._double(self.solver.StartTime)
            end_time = self._double(self.solver.StopTime)

            if base_id:
                triad_id = base_id
            else:
                triad = self.fedApp.FindByDescription(location)
                triad_id = str(triad.BaseID)
            #print(triad_id)

            FRS = self.solver.GetFRSFileNames
            resEX = self.fedApp.CreateResultExtractor(FRS)
            resEX.ReadTime = True

            data = resEX.Read(
                'Triad',
                triad_id,
                field,
                start_time,
                end_time)

            if not data:
                return pd.DataFrame()

            df = pd.DataFrame((val for _, val in data),
                              index=(t for t, _ in data),
                              columns=['{}:{}'.format(location,
                                                      field)],
                              )
            df.index.name = 'time'

            return df

        except pythoncom.com_error as error:
            print (error)
            print (vars(error))
            print (error.args)

    # x_pos = _get_results('Position matrix:TMAT34:Position X')
    # z_pos = _get_results('Position matrix:TMAT34:Position Z')
    #
    # def get_result_data(*args,**kwargs):
    #     data = {'time': [d[0] for d in x_pos],
    #             'x': [d[1] for d in x_pos],
    #             'y': [d[1] for d in z_pos]}
    #     data = pd.DataFrame(data).set_index('time')
    #     return data

    @staticmethod
    def _fedems():
        context = pythoncom.CreateBindCtx()
        rot = pythoncom.GetRunningObjectTable()

        for moniker in rot:
            name = moniker.GetDisplayName(context, None)
            if "!{C0D470AF-77B3-4880-864D-E5FF" in name:
                obj = rot.GetObject(moniker)
                fedApp = win32com.client.Dispatch(obj.QueryInterface(pythoncom.IID_IDispatch))
                print("Found", name)
                yield fedApp

    @staticmethod
    def _fedem_at_file(f):
        popen_res = subprocess.Popen(['start', f], shell=True)
        for _ in range(60):
            for fedapp in Fedem._fedems():
                print('Fedem at file:', fedapp.ModelFileName)
                if fedapp.ModelFileName == f:
                    print('found right file')
                    return fedapp

            print('Did not find FEDEM w', f)
            time.sleep(1.0)

        print('Popen res:', popen_res)
        raise ValueError('No fedem w file: '+f)



if __name__ == '__main__':
    path = 'C:\\Users\\arnt\\Documents\\GitHub\\KBE\\library\\mechtimum-files\\out\\'
    file = 'model.fmm'

    fedapp = Fedem(path, file)
    fedapp.end()

