from pprint import pprint
from jinja2 import Environment, FileSystemLoader

env = Environment(
    loader=FileSystemLoader('./templates'),
    trim_blocks=True,
)

def num_to_str(num):
    return '{:0.5f}'.format(num)

def qstr(s):
    return '"{}"'.format(s)

def arr(vec):
    return '({})'.format(' '.join(num_to_str(x) for x in vec))

env.globals.update(num_to_str=num_to_str,
                   qstr=qstr,
                   arr=arr)


def write_template(path, file, *args,  **kwargs):
    with open(path+file, 'w') as fp:
        fp.write(env.get_template(file).render(*args, **kwargs))

def write_files(dict, path='C:/Users/arnt/Documents/GitHub/KBE/library/mechtimum-files/out/'):
    points = dict['points']
    point_data = sorted(points.values(), key=lambda p: p['id'])

    links = dict['links']
    members = [m for l in links.values() for m in l.values()]

    constraints = dict['constraints']
    constraint_data = sorted(constraints.values(),
                             key=lambda c: c['point'])
    for e in constraint_data:
        e['links'] = '({})'.format(' '.join((['nil'] + [str(i) for i in e['links']])[-2:]))
        e['joint-dir'] = '({})'.format(' '.join(num_to_str(i) for i in e['joint-dir']))
        e['fixed-dofs'] = '({})'.format(' '.join([str(i) for i in e['fixed-dofs']])) if e['fixed-dofs'] else ''

    loads = dict['loads'].values()

    couplings = dict['couplings'].values()
    print(loads)
    for c in couplings:
        c['links'] = '({})'.format(' '.join([str(id if id else 'nil') for id in c['links']]))

    write_template(path, 'coordinates.txt',
                   items=point_data, delim='\t' * 3)
    write_template(path, 'shapes.txt',
                   items=members, delim='\t' * 3)
    write_template(path, 'constraints.txt',
                   items=constraint_data, delim='\t' * 3)
    write_template(path, 'loads.txt',
                   items=loads, delim='\t'*3)
    write_template(path, 'spring-damper.txt',
                   items=couplings, delim='\t'*3)
    write_template(path, 'optimization.txt',
                   items=couplings, delim='\t'*3)



