from __future__ import print_function

import sys
import numpy as np

from openmdao.api import Group, Component, MetaModel, KrigingSurrogate, FloatKrigingSurrogate
from openmdao.api import Problem


class TrigMM(Group):
    ''' FloatKriging gives responses as floats '''

    def __init__(self):
        super(TrigMM, self).__init__()

        # Create meta_model for f_x as the response
        sin_mm = self.add("sin_mm", MetaModel())
        sin_mm.add_param('x1', val=0.)
        sin_mm.add_param('x2', val=0.)
        sin_mm.add_output('f', val=0., surrogate=FloatKrigingSurrogate())

prob = Problem()
prob.root = TrigMM()
prob.setup()

#traning data is just set manually. No connected input needed, since
#  we're assuming the data is pre-existing
prob['sin_mm.train:x1'] = 0.5 * np.linspace(0, 10, 20)
prob['sin_mm.train:x2'] = 0.5 * np.linspace(0, 10, 20)
prob['sin_mm.train:f'] = np.sin(prob['sin_mm.train:x1'] + prob['sin_mm.train:x2'])

prob['sin_mm.x1'] = np.array([1.0]) #prediction happens at this value
prob['sin_mm.x2'] = np.array([1.1]) #prediction happens at this value
prob.run()
print('float predicted:', prob['sin_mm.f']) #predicted value
print('float actual: ', np.sin(2.1))
