from openmdao.api import Problem, view_model, Component, view_connections, IndepVarComp, ScipyOptimizer, SqliteRecorder

from yamlparser import parsefile
from file_writer import write_files
from Aml import FmmWriter
from fedem import Fedem

import time

class Analyser(Component):

    def __init__(self):
        super().__init__()

        self.aml = FmmWriter()

        self.add_param('dict',val='', pass_by_obj=True)
        self.add_output('res', shape=1)

    def solve_nonlinear(self, params, unknowns, resids):
        print('DOING THE DEFEM')
        path = 'C:\\Users\\arnt\\Documents\\GitHub\\KBE\\library\\mechtimum-files\\out\\'
        fmm_file = 'model.fmm'

        write_files(params['dict'], path=path)
        self.aml.write_fmm()
        time.sleep(1.0)
        fedem = Fedem(path, fmm_file)
        v = fedem.get_res_on_bid(9, 'Velocity:VEC3:Y', 'vel')

        result = fedem.get_res_on_bid(9, 'Position matrix:TMAT34:Position X', 'x')
        result.columns = ['x']
        print(result.head())
        not_of_interest = result.loc[lambda df: df.x < 0, :]
        settle_time = not_of_interest.index[0]
        obj = 1000000.0 * abs(settle_time-2.0)
        print('OBJ:', "{0:.6f}".format(settle_time))
        print('ABS:', "{0:.6f}".format(obj))
        fedem.end()
        unknowns['res'] = obj
        print('DONE THE DEFEM')


g, d = parsefile('four-bar.yaml')
analyser = Analyser()
g.add('analyser', analyser, promotes=['dict', 'res'])
prob = Problem(root=g)
prob.root.add('design_param', IndepVarComp('c', 2005.0))
prob.root.connect('design_param.c', 'couplings.NodeCoupling0.constant')


prob.root.deriv_options['type'] = 'fd'
prob.root.deriv_options['form'] = 'backward'
prob.root.deriv_options['step_size'] = 50.0

prob.driver = ScipyOptimizer()
prob.driver.options['optimizer'] = 'SLSQP'
prob.driver.options['tol'] = 1e-2
prob.driver.add_desvar('design_param.c', lower=400.0, upper=4000.0)
prob.driver.add_objective('res')

recorder = SqliteRecorder('log1')
recorder.options['record_params'] = True
recorder.options['record_metadata'] = True
recorder.options['record_derivs'] = True
prob.driver.add_recorder(recorder)

prob.setup()
#view_model(prob)
#view_connections(g)
print('running')

prob.run()
print('the result is ', prob['res'], 'at ', prob['design_param.c'])

analyser.aml.aml.end()
prob.cleanup()

