from pyNastran.bdf.bdf import BDF
import numpy as np
import pandas as pd
import heapq
import itertools
import math


options = '''
GAGEOPTIONS
{
AUTO_DAC_EXPORT = false;
DAC_SAMPLE_RATE = 0.001;
ID = 1;
RAINFLOW_ANALYSIS = false;
START_TIME = 0;
STOP_TIME = 100;
STRESS_BINSIZE = 10000000;
TIME_INCR = 0.01;
USE_ALL_TIME_STEPS = true;
}
'''


def prep_fmm_for_gauge(fmm_path):
    contents = []

    with open(fmm_path, 'r+') as fd:
        for line in fd:
            if line.startswith('REF_PLANE'):
                contents.append(options)

            contents.append(line)

        fd.seek(0)
        fd.truncate()
        fd.writelines(contents)

def create_strain_rosette(fedem, bdf_path):
    plan_tol = 0.08
    start = np.array([0.0, 0.75, 0.0])
    end = np.array([1.5, 1.875, 0.0])
    base_length = 0.2
    direction = np.array([-1.0, 1.0, 0.0])
    dir_len = np.linalg.norm(direction)
    bdf_filename = bdf_path
    nodes = find_straine_rose_nodes(plan_tol, start, end, base_length, direction, dir_len, bdf_filename)

    fedem.create_strain_rosette(nodes, 6)


def find_straine_rose_nodes(plan_tol, start, end, base_length, direction, dir_len, bdf_filename):
    #global model, nodes
    # finding the point
    plan_normal = base_length * (end - start)
    #print(plan_normal)
    base = start + plan_normal
    #print(base)
    plan_normal_abs = np.linalg.norm(base)
    #print(plan_normal_abs)
    model = BDF()
    model.read_bdf(bdf_filename)
    #print(model.get_bdf_stats())
    planar_distances = []
    for elem_id, elem in model.elements.items():
        coa = sum([n.xyz for n in elem.nodes]) / len(elem.nodes)
        dist = np.abs(np.dot(plan_normal, -(coa - base))) / plan_normal_abs
        planar_distances.append((elem_id, dist, coa))
    planar_distances = pd.DataFrame(planar_distances,
                                    columns=['elem_id', 'distance', 'coa'])
    planar_distances.head()
    p1 = planar_distances.sort_values('distance')
    p2 = planar_distances.sort_values('distance', ascending=False)
    #print(planar_distances.head(20))
    #print(p1)
    #print('*' * 100)
    print(planar_distances.min())
    nearest_elem_id = planar_distances.min().loc['elem_id']
    print('neares elem is ', nearest_elem_id)
    nearest_elem = model.elements[nearest_elem_id]
    print(nearest_elem)
    size = np.max([np.linalg.norm(a.xyz-b.xyz)
                           for a, b
                           in itertools.combinations(nearest_elem.nodes, r=2)])

    print('the elem size is about:', size)
    for n in nearest_elem.nodes:
        print(n.xyz)

    print([np.linalg.norm(a.xyz-b.xyz)
           for a, b
           in itertools.combinations(nearest_elem.nodes, r=2)])

    plan_tol = size

    p2 = planar_distances.loc[lambda df: df.distance < plan_tol].copy()
    #print('p2', len(p2), p2)
    #print('-' * 100)
    f = lambda coa: np.dot(coa, direction) / dir_len
    p2['line_dist'] = p2['coa'].apply(f)
    #print('p2:\n', p2)
    #print(':' * 100)
    idx = p2['line_dist'].idxmax()
    #print('idx', idx)
    elem_id = p2.loc[idx, 'elem_id']
    elem_coa = p2.loc[idx, 'coa']
    the_elem = model.elements[elem_id]
    #print('the elem:', the_elem)
    #print('elem pos: ', elem_coa)
    #print('node posistions:')
    #for n in the_elem.nodes:
        #print('node at: ', n.xyz)
        #print(n)
        #print('-' * 30)
    nodal_dist = {node.nid: f(node.xyz) for node in the_elem.nodes}
    #print(nodal_dist)
    nodes = heapq.nlargest(4, nodal_dist, nodal_dist.get)
    return nodes[:3]

if __name__ == '__main__':
    #From problem def
    plan_tol = 0.005
    start = np.array([0.0, 0.0, 0.0])
    end = np.array([0.0, 0.75, 0.0])
    base_length = 0.4
    direction = np.array([-1.0, 0.0, 0.0])
    dir_len = np.linalg.norm(direction)
    bdf_filename = 'C:\\Temp\\nastran-data\\mechtimum-files - out\\link-0000.bdf'

    #From problem def
    res = find_straine_rose_nodes(plan_tol, start, end, base_length, direction, dir_len, bdf_filename)
    print(res)
    # #print(nodes)
    # v0 = model.nodes[nodes[2]].xyz
    # v1 = v0 - model.nodes[nodes[0]].xyz
    # v2 = v0 - model.nodes[nodes[1]].xyz
    # v = np.cross(v1,v2)
    # #print(v/np.linalg.norm(v))