from openmdao.api import Problem, view_model, Component, view_connections, IndepVarComp, ScipyOptimizer, SqliteRecorder
import numpy as np

from yamlparser import parsefile
from file_writer import write_files
from Aml import FmmWriter
from fedem import Fedem
from strain_rose import find_straine_rose_nodes

import time


def prep_fmm_for_gauge(fmm_path):
    options = '''
GAGEOPTIONS
{
AUTO_DAC_EXPORT = false;
DAC_SAMPLE_RATE = 0.001;
ID = 1;
RAINFLOW_ANALYSIS = false;
START_TIME = 0;
STOP_TIME = 100;
STRESS_BINSIZE = 10000000;
TIME_INCR = 0.01;
USE_ALL_TIME_STEPS = true;
}
'''

    contents = []

    with open(fmm_path, 'r+') as fd:
        for line in fd:
            if line.startswith('REF_PLANE'):
                contents.append(options)

            contents.append(line)

        fd.seek(0)
        fd.truncate()
        fd.writelines(contents)


def create_starin_rosette(fedem, bdf_path):
    bdf_path = fedem.get_bfd_path(3)
    plan_tol = 0.005
    start = np.array([0.0, 0.0, 0.0])
    end = np.array([0.0, 0.75, 0.0])
    base_length = 0.4
    direction = np.array([-1.0, 0.0, 0.0])
    dir_len = np.linalg.norm(direction)
    bdf_filename = bdf_path
    nodes = find_straine_rose_nodes(plan_tol, start, end, base_length, direction, dir_len, bdf_filename)

    fedem.create_strain_rosette(nodes, 3)


print('DOING THE DEFEM')
path = 'C:\\Users\\arnt\\Documents\\GitHub\\KBE\\library\\mechtimum-files\\out\\'
bdf_filename = 'C:\\Temp\\nastran-data\\mechtimum-files - out\\link-0000.bdf'
fmm_file = 'model.fmm'

#prep_fmm_for_gauge(path + fmm_file)
fedem = Fedem(path, fmm_file)

create_starin_rosette(fedem, bdf_filename)

fedem.run(onlydyn=False, result_check_args=('rose', 'Stress tensor:TENSOR2:Von Mises', 29))

result = None
while True:
    result = fedem.get_res_on_bid(29, 'Stress tensor:TENSOR2:Von Mises', 'theRose')
    #print('index:')
    #print(result.index)
    #print(result)
    #print('the value', result.iloc[-1].loc['theRose:Stress tensor:TENSOR2:Von Mises'])
    #exit()
    print(result.tail(3))
    if result.iloc[-1].loc['theRose:Stress tensor:TENSOR2:Von Mises'] != 0.0:
        print(result.tail())
        break
    time.sleep(0.5)

print(result)
